@@ -6,14 +6,10 @@ import uuid
 import nbformat
 import papermill
 import six
-from papermill.engines import papermill_engines
-from papermill.iorw import load_notebook_node, write_ipynb
-from papermill.parameterize import _find_first_tagged_cell_index
-
 from dagster import (
+    AssetMaterialization,
     EventMetadataEntry,
     InputDefinition,
-    Materialization,
     Output,
     OutputDefinition,
     SolidDefinition,
@@ -28,7 +24,9 @@ from dagster.core.execution.context.system import SystemComputeExecutionContext
 from dagster.core.storage.file_manager import FileHandle
 from dagster.serdes import pack_value
 from dagster.utils import mkdir_p, safe_tempfile_path
-from dagster.utils.backcompat import canonicalize_backcompat_args
+from papermill.engines import papermill_engines
+from papermill.iorw import load_notebook_node, write_ipynb
+from papermill.parameterize import _find_first_tagged_cell_index
 
 from .engine import DagstermillNBConvertEngine
 from .errors import DagstermillError, DagstermillExecutionError
@@ -40,13 +38,13 @@ from .translator import RESERVED_INPUT_NAMES, DagsterTranslator
 # Typically, papermill injects the injected-parameters cell *below* the parameters cell
 # but we want to *replace* the parameters cell, which is what this function does.
 def replace_parameters(context, nb, parameters):
-    '''Assigned parameters into the appropiate place in the input notebook
+    """Assigned parameters into the appropiate place in the input notebook
 
     Args:
         nb (NotebookNode): Executable notebook object
         parameters (dict): Arbitrary keyword arguments to pass to the notebook parameters.
-    '''
-    check.dict_param(parameters, 'parameters')
+    """
+    check.dict_param(parameters, "parameters")
 
     # Copy the nb object to avoid polluting the input
     nb = copy.deepcopy(nb)
@@ -56,15 +54,15 @@ def replace_parameters(context, nb, parameters):
     param_content = DagsterTranslator.codify(parameters)
 
     newcell = nbformat.v4.new_code_cell(source=param_content)
-    newcell.metadata['tags'] = ['injected-parameters']
+    newcell.metadata["tags"] = ["injected-parameters"]
 
-    param_cell_index = _find_first_tagged_cell_index(nb, 'parameters')
-    injected_cell_index = _find_first_tagged_cell_index(nb, 'injected-parameters')
+    param_cell_index = _find_first_tagged_cell_index(nb, "parameters")
+    injected_cell_index = _find_first_tagged_cell_index(nb, "injected-parameters")
     if injected_cell_index >= 0:
         # Replace the injected cell with a new version
         before = nb.cells[:injected_cell_index]
         after = nb.cells[injected_cell_index + 1 :]
-        check.int_value_param(param_cell_index, -1, 'param_cell_index')
+        check.int_value_param(param_cell_index, -1, "param_cell_index")
         # We should have blown away the parameters cell if there is an injected-parameters cell
     elif param_cell_index >= 0:
         # Replace the parameter cell with the injected-parameters cell
@@ -74,45 +72,45 @@ def replace_parameters(context, nb, parameters):
         # Inject to the top of the notebook, presumably first cell includes dagstermill import
         context.log.debug(
             (
-                'Executing notebook with no tagged parameters cell: injecting boilerplate in first '
-                'cell.'
+                "Executing notebook with no tagged parameters cell: injecting boilerplate in first "
+                "cell."
             )
         )
-        before = nb.cells[:1]
-        after = nb.cells[1:]
+        before = []
+        after = nb.cells
 
     nb.cells = before + [newcell] + after
-    nb.metadata.papermill['parameters'] = seven.json.dumps(parameters)
+    nb.metadata.papermill["parameters"] = seven.json.dumps(parameters)
 
     return nb
 
 
 def get_papermill_parameters(compute_context, inputs, output_log_path):
-    check.inst_param(compute_context, 'compute_context', SystemComputeExecutionContext)
+    check.inst_param(compute_context, "compute_context", SystemComputeExecutionContext)
     check.param_invariant(
         isinstance(compute_context.run_config, dict),
-        'compute_context',
-        'SystemComputeExecutionContext must have valid run_config',
+        "compute_context",
+        "SystemComputeExecutionContext must have valid run_config",
     )
-    check.dict_param(inputs, 'inputs', key_type=six.string_types)
+    check.dict_param(inputs, "inputs", key_type=six.string_types)
 
     run_id = compute_context.run_id
 
-    marshal_dir = '/tmp/dagstermill/{run_id}/marshal'.format(run_id=run_id)
+    marshal_dir = "/tmp/dagstermill/{run_id}/marshal".format(run_id=run_id)
     mkdir_p(marshal_dir)
 
     if not isinstance(compute_context.pipeline, ReconstructablePipeline):
         raise DagstermillError(
-            'Can\'t execute a dagstermill solid from a pipeline that is not reconstructable. '
-            'Use the reconstructable() function if executing from python'
+            "Can't execute a dagstermill solid from a pipeline that is not reconstructable. "
+            "Use the reconstructable() function if executing from python"
         )
 
     dm_executable_dict = compute_context.pipeline.to_dict()
 
     dm_context_dict = {
-        'output_log_path': output_log_path,
-        'marshal_dir': marshal_dir,
-        'run_config': compute_context.run_config,
+        "output_log_path": output_log_path,
+        "marshal_dir": marshal_dir,
+        "run_config": compute_context.run_config,
     }
 
     dm_solid_handle_kwargs = compute_context.solid_handle._asdict()
@@ -123,33 +121,33 @@ def get_papermill_parameters(compute_context, inputs, output_log_path):
     for input_name, input_value in inputs.items():
         assert (
             input_name not in RESERVED_INPUT_NAMES
-        ), 'Dagstermill solids cannot have inputs named {input_name}'.format(input_name=input_name)
+        ), "Dagstermill solids cannot have inputs named {input_name}".format(input_name=input_name)
         dagster_type = input_def_dict[input_name].dagster_type
         parameter_value = write_value(
-            dagster_type, input_value, os.path.join(marshal_dir, 'input-{}'.format(input_name))
+            dagster_type, input_value, os.path.join(marshal_dir, "input-{}".format(input_name))
         )
         parameters[input_name] = parameter_value
 
-    parameters['__dm_context'] = dm_context_dict
-    parameters['__dm_executable_dict'] = dm_executable_dict
-    parameters['__dm_pipeline_run_dict'] = pack_value(compute_context.pipeline_run)
-    parameters['__dm_solid_handle_kwargs'] = dm_solid_handle_kwargs
-    parameters['__dm_instance_ref_dict'] = pack_value(compute_context.instance.get_ref())
+    parameters["__dm_context"] = dm_context_dict
+    parameters["__dm_executable_dict"] = dm_executable_dict
+    parameters["__dm_pipeline_run_dict"] = pack_value(compute_context.pipeline_run)
+    parameters["__dm_solid_handle_kwargs"] = dm_solid_handle_kwargs
+    parameters["__dm_instance_ref_dict"] = pack_value(compute_context.instance.get_ref())
 
     return parameters
 
 
 def _dm_solid_compute(name, notebook_path, output_notebook=None):
-    check.str_param(name, 'name')
-    check.str_param(notebook_path, 'notebook_path')
-    check.opt_str_param(output_notebook, 'output_notebook')
+    check.str_param(name, "name")
+    check.str_param(notebook_path, "notebook_path")
+    check.opt_str_param(output_notebook, "output_notebook")
 
     def _t_fn(compute_context, inputs):
-        check.inst_param(compute_context, 'compute_context', SolidExecutionContext)
+        check.inst_param(compute_context, "compute_context", SolidExecutionContext)
         check.param_invariant(
             isinstance(compute_context.run_config, dict),
-            'context',
-            'SystemComputeExecutionContext must have valid run_config',
+            "context",
+            "SystemComputeExecutionContext must have valid run_config",
         )
 
         system_compute_context = compute_context.get_system_context()
@@ -158,11 +156,11 @@ def _dm_solid_compute(name, notebook_path, output_notebook=None):
             with safe_tempfile_path() as output_log_path:
 
                 parameterized_notebook_path = os.path.join(
-                    output_notebook_dir, '{prefix}-inter.ipynb'.format(prefix=str(uuid.uuid4()))
+                    output_notebook_dir, "{prefix}-inter.ipynb".format(prefix=str(uuid.uuid4()))
                 )
 
                 executed_notebook_path = os.path.join(
-                    output_notebook_dir, '{prefix}-out.ipynb'.format(prefix=str(uuid.uuid4()))
+                    output_notebook_dir, "{prefix}-out.ipynb".format(prefix=str(uuid.uuid4()))
                 )
 
                 # Scaffold the registration here
@@ -177,41 +175,41 @@ def _dm_solid_compute(name, notebook_path, output_notebook=None):
                 with user_code_error_boundary(
                     DagstermillExecutionError,
                     lambda: (
-                        'Error occurred during the execution of Dagstermill solid '
-                        '{solid_name}: {notebook_path}'.format(
+                        "Error occurred during the execution of Dagstermill solid "
+                        "{solid_name}: {notebook_path}".format(
                             solid_name=name, notebook_path=notebook_path
                         )
                     ),
                 ):
                     try:
-                        papermill_engines.register('dagstermill', DagstermillNBConvertEngine)
+                        papermill_engines.register("dagstermill", DagstermillNBConvertEngine)
                         papermill.execute_notebook(
                             input_path=parameterized_notebook_path,
                             output_path=executed_notebook_path,
-                            engine_name='dagstermill',
+                            engine_name="dagstermill",
                             log_output=True,
                         )
 
                     except Exception as exc:  # pylint: disable=broad-except
                         try:
-                            with open(executed_notebook_path, 'r') as fd:
+                            with open(executed_notebook_path, "rb") as fd:
                                 executed_notebook_file_handle = compute_context.file_manager.write(
-                                    fd, mode='w', ext='ipynb'
+                                    fd, mode="wb", ext="ipynb"
                                 )
                                 executed_notebook_materialization_path = (
                                     executed_notebook_file_handle.path_desc
                                 )
                         except Exception as exc_inner:  # pylint: disable=broad-except
                             compute_context.log.warning(
-                                'Error when attempting to materialize executed notebook using file manager (falling back to local): {exc}'.format(
+                                "Error when attempting to materialize executed notebook using file manager (falling back to local): {exc}".format(
                                     exc=exc_inner
                                 )
                             )
                             executed_notebook_materialization_path = executed_notebook_path
 
-                        yield Materialization(
-                            label='output_notebook',
-                            description='Location of output notebook in file manager',
+                        yield AssetMaterialization(
+                            asset_key=executed_notebook_materialization_path,
+                            description="Location of output notebook in file manager",
                             metadata_entries=[
                                 EventMetadataEntry.fspath(executed_notebook_materialization_path)
                             ],
@@ -219,28 +217,31 @@ def _dm_solid_compute(name, notebook_path, output_notebook=None):
                         raise exc
 
             system_compute_context.log.debug(
-                'Notebook execution complete for {name} at {executed_notebook_path}.'.format(
+                "Notebook execution complete for {name} at {executed_notebook_path}.".format(
                     name=name, executed_notebook_path=executed_notebook_path,
                 )
             )
 
+            executed_notebook_file_handle = None
             try:
-                with open(executed_notebook_path, 'r') as fd:
+                # use binary mode when when moving the file since certain file_managers such as S3
+                # may try to hash the contents
+                with open(executed_notebook_path, "rb") as fd:
                     executed_notebook_file_handle = compute_context.file_manager.write(
-                        fd, mode='w', ext='ipynb'
+                        fd, mode="wb", ext="ipynb"
                     )
                     executed_notebook_materialization_path = executed_notebook_file_handle.path_desc
             except Exception as exc:  # pylint: disable=broad-except
                 compute_context.log.warning(
-                    'Error when attempting to materialize executed notebook using file manager (falling back to local): {exc}'.format(
+                    "Error when attempting to materialize executed notebook using file manager (falling back to local): {exc}".format(
                         exc=str(exc)
                     )
                 )
                 executed_notebook_materialization_path = executed_notebook_path
 
-            yield Materialization(
-                label='output_notebook',
-                description='Location of output notebook in file manager',
+            yield AssetMaterialization(
+                asset_key=executed_notebook_materialization_path,
+                description="Location of output notebook in file manager",
                 metadata_entries=[
                     EventMetadataEntry.fspath(executed_notebook_materialization_path)
                 ],
@@ -262,8 +263,8 @@ def _dm_solid_compute(name, notebook_path, output_notebook=None):
                     yield Output(value, output_name)
 
             for key, value in output_nb.scraps.items():
-                if key.startswith('event-'):
-                    with open(value.data, 'rb') as fd:
+                if key.startswith("event-"):
+                    with open(value.data, "rb") as fd:
                         yield pickle.loads(fd.read())
 
     return _t_fn
@@ -274,12 +275,11 @@ def define_dagstermill_solid(
     notebook_path,
     input_defs=None,
     output_defs=None,
-    config=None,
+    config_schema=None,
     required_resource_keys=None,
     output_notebook=None,
-    config_schema=None,
 ):
-    '''Wrap a Jupyter notebook in a solid.
+    """Wrap a Jupyter notebook in a solid.
 
     Arguments:
         name (str): The name of the solid.
@@ -290,20 +290,20 @@ def define_dagstermill_solid(
         required_resource_keys (Optional[Set[str]]): The string names of any required resources.
         output_notebook (Optional[str]): If set, will be used as the name of an injected output of
             type :py:class:`~dagster.FileHandle` that will point to the executed notebook (in
-            addition to the :py:class:`~dagster.Materialization` that is always created). This
+            addition to the :py:class:`~dagster.AssetMaterialization` that is always created). This
             respects the :py:class:`~dagster.core.storage.file_manager.FileManager` configured on
             the pipeline system storage, so, e.g., if :py:class:`~dagster_aws.s3.s3_system_storage`
             is configured, the output will be a :py:class:`~dagster_aws.s3.S3FileHandle`.
 
     Returns:
         :py:class:`~dagster.SolidDefinition`
-    '''
-    check.str_param(name, 'name')
-    check.str_param(notebook_path, 'notebook_path')
-    input_defs = check.opt_list_param(input_defs, 'input_defs', of_type=InputDefinition)
-    output_defs = check.opt_list_param(output_defs, 'output_defs', of_type=OutputDefinition)
+    """
+    check.str_param(name, "name")
+    check.str_param(notebook_path, "notebook_path")
+    input_defs = check.opt_list_param(input_defs, "input_defs", of_type=InputDefinition)
+    output_defs = check.opt_list_param(output_defs, "output_defs", of_type=OutputDefinition)
     required_resource_keys = check.opt_set_param(
-        required_resource_keys, 'required_resource_keys', of_type=str
+        required_resource_keys, "required_resource_keys", of_type=str
     )
 
     return SolidDefinition(
@@ -316,14 +316,8 @@ def define_dagstermill_solid(
             if output_notebook
             else []
         ),
-        config_schema=canonicalize_backcompat_args(
-            check_user_facing_opt_config_param(config_schema, 'config_schema'),
-            'config_schema',
-            check_user_facing_opt_config_param(config, 'config'),
-            'config',
-            '0.9.0',
-        ),
+        config_schema=check_user_facing_opt_config_param(config_schema, "config_schema"),
         required_resource_keys=required_resource_keys,
-        description='This solid is backed by the notebook at {path}'.format(path=notebook_path),
-        tags={'notebook_path': notebook_path, 'kind': 'ipynb'},
+        description="This solid is backed by the notebook at {path}".format(path=notebook_path),
+        tags={"notebook_path": notebook_path, "kind": "ipynb"},
     )
